@@ -3,7 +3,7 @@ import csv
 from Bio import SeqIO
 from collections import defaultdict
 from annogesiclib.gff3 import Gff3Parser, Gff3Entry
-from annogesiclib.TSSpredator import TSSPredatorReader
+from annogesiclib.TSSpredator_parser import TSSPredatorReader
 from annogesiclib.helper import Helper
 
 
@@ -14,11 +14,29 @@ class Converter(object):
         self.gff3parser = Gff3Parser()
         self.tssparser = TSSPredatorReader()
 
+    def _check_locus_tag(self, entry, genes):
+        gene_tag = "-"
+        locus_tag = "-"
+        if "locus_tag" in entry.attributes.keys():
+            locus_tag = entry.attributes["locus_tag"]
+        elif "Parent" in entry.attributes.keys():
+            for gene in genes:
+                if (gene.attributes["ID"] in
+                        entry.attributes["Parent"].split(",")):
+                    if "gene" in gene.attributes.keys():
+                        gene_tag = gene.attributes["gene"]
+                    if "locus_tag" in gene.attributes.keys():
+                        locus_tag = gene.attributes["locus_tag"]
+        if locus_tag == "-":
+            locus_tag = "".join([
+                entry.feature, ":", str(entry.start), "-",
+                str(entry.end), "_", entry.strand])
+        return locus_tag, gene_tag
+
+
     def _print_rntptt_file(self, out, entrys, genes):
         '''output to rnt and ptt file'''
         for entry in entrys:
-            gene_tag = "-"
-            locus_tag = "-"
             location = "..".join([str(entry.start), str(entry.end)])
             length = str(entry.end - entry.start + 1)
             if entry.feature == "CDS":
@@ -26,35 +44,11 @@ class Converter(object):
                     pid = entry.attributes["protein_id"]
                 else:
                     pid = "-"
-                if "locus_tag" in entry.attributes.keys():
-                    locus_tag = entry.attributes["locus_tag"]
-                elif "Parent" in entry.attributes.keys():
-                    for gene in genes:
-                        if (gene.attributes["ID"] in 
-                                entry.attributes["Parent"].split(",")):
-                            if "gene" in gene.attributes.keys():
-                                gene_tag = gene.attributes["gene"]
-                            locus_tag = gene.attributes["locus_tag"]
-                else:
-                    locus_tag = "".join([
-                        entry.feature, ":", str(entry.start), "-",
-                        str(entry.end), "_", entry.strand])
+                gene_tag, locus_tag = self._check_locus_tag(entry, genes)
             else:
                 pid = "-"
                 gene_tag = "-"
-                if "locus_tag" in entry.attributes.keys():
-                    locus_tag = entry.attributes["locus_tag"]
-                elif "Parent" in entry.attributes.keys():
-                    for gene in genes:
-                        if (gene.attributes["ID"] in 
-                                entry.attributes["Parent"].split(",")):
-                            if "gene" in gene.attributes.keys():
-                                gene_tag = gene.attributes["gene"]
-                            locus_tag = gene.attributes["locus_tag"]
-                else:
-                    locus_tag = "".join([
-                        entry.feature, ":", str(entry.start), "-",
-                        str(entry.end), "_", entry.strand])
+                gene_tag, locus_tag = self._check_locus_tag(entry, genes)
             if "product" in entry.attributes.keys():
                 product = entry.attributes["product"]
             else:
@@ -70,30 +64,38 @@ class Converter(object):
         out.write("\t".join(["Location", "Strand", "Length", "PID",
                   "Gene", "Synonym", "Code", "COG", "Product"]) + "\n")
 
-    def _read_file(self, gff_file, fasta_file, rnas, cdss, genes):
+    def _read_file(self, gff_file, fasta_file, rnas, cdss, genes, seq_id):
         num_cds = 0
         num_rna = 0
         seq = ""
         g_f = open(gff_file, "r")
         for entry in self.gff3parser.entries(g_f):
-            if (entry.feature == "rRNA") or (entry.feature == "tRNA"):
-                num_rna += 1
-                rnas.append(entry)
-            elif entry.feature == "CDS":
-                num_cds += 1
-                cdss.append(entry)
-            elif entry.feature == "gene":
-                genes.append(entry)
+            if entry.seq_id == seq_id:
+                if (entry.feature == "rRNA") or (entry.feature == "tRNA"):
+                    num_rna += 1
+                    rnas.append(entry)
+                elif entry.feature == "CDS":
+                    num_cds += 1
+                    cdss.append(entry)
+                elif entry.feature == "gene":
+                    genes.append(entry)
         g_f.close()
         if fasta_file == "0":
             seq = "-1"
         else:
+            detect = False
             with open(fasta_file, "r") as f_f:
                 for line in f_f:
                     line = line.strip()
                     if len(line) != 0:
-                        if line[0] != ">":
-                            seq = seq + line
+                        if line.startswith(">"):
+                            if line[1:] == seq_id:
+                                detect = True
+                            else:
+                                detect = False
+                        else:
+                            if detect:
+                                seq = seq + line
         return (num_cds, num_rna, seq)
 
     def _srna2rntptt(self, srna_input_file, srna_output_file, srnas, length):
@@ -294,14 +296,14 @@ class Converter(object):
                              str(tss.super_pos), ".", tss.super_strand, ".",
                              attribute_string]) + "\n")
 
-    def convert_gff2rntptt(self, gff_file, fasta_file, ptt_file, rnt_file,
+    def convert_gff2rntptt(self, gff_file, seq_id, fasta_file, ptt_file, rnt_file,
                            srna_input_file, srna_output_file):
         '''Convert gff format to rnt and ptt format'''
         genes = []
         rnas = []
         cdss = []
         srnas = []
-        datas = self._read_file(gff_file, fasta_file, rnas, cdss, genes)
+        datas = self._read_file(gff_file, fasta_file, rnas, cdss, genes, seq_id)
         rnas = sorted(rnas, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
         cdss = sorted(cdss, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
         genes = sorted(genes, key=lambda k: (k.seq_id, k.start,
