@@ -1,4 +1,5 @@
 import math
+import shutil
 import matplotlib as mpl
 from annogesiclib.gff3 import Gff3Parser
 from annogesiclib.helper import Helper
@@ -14,16 +15,19 @@ def plot(utr, utr_pri, utr_sec, filename, source, utr_type, base_5utr):
     if utr_type == "5utr":
         if source and (base_5utr != "transcript"):
             n, bins, hist1 = plt.hist(utr, bin_num,
-                                      color="#FF9999", label='secondary')
+                                      color="#FF9999", label='secondary',
+                                      edgecolor='black', linewidth=1)
             n, bins, hist2 = plt.hist(utr_pri, bin_num,
-                                      color="#9999FF", label='primary')
+                                      color="#9999FF", label='primary',
+                                      edgecolor='black', linewidth=1)
             plt.legend((hist2[0], hist1[0]),
                        ("Primary TSSs", "Secondary TSSs"))
         else:
             n, bins, hist1 = plt.hist(utr, bin_num, color="#FF9999")
         plt.xlabel("5'UTR_length")
     elif utr_type == "3utr":
-        n, bins, hist = plt.hist(utr, bin_num, color="#9999FF", label='3\'UTR')
+        n, bins, hist = plt.hist(utr, bin_num, color="#9999FF", label='3\'UTR',
+                                 edgecolor='black', linewidth=1)
         plt.legend([hist[0]], ["3'UTR"])
         plt.xlabel("3'UTR_length")
     plt.ylabel("Amount")
@@ -88,7 +92,10 @@ def import_utr(tss, utr_strain, utr_all, start, end, tas, length, args_utr):
 
 
 def get_print_string_5utr(num_utr, name_utr, length, tss, cds_name,
-                          locus_tag, ta, source, out, start, end):
+                          locus_tag, ta, source, out, start, end, utrs_tss):
+    if "Name" not in tss.attributes.keys():
+        tss.attributes["Name"] = (tss.feature + ":" + str(tss.start) + "-" + 
+                                  str(tss.end) + "_" + tss.strand)
     attribute_string = ";".join(
              ["=".join(items) for items in [
               ("ID", "_".join([tss.seq_id, "utr5", str(num_utr)])),
@@ -111,28 +118,43 @@ def get_print_string_5utr(num_utr, name_utr, length, tss, cds_name,
     out.write("{0}\tANNOgesic\t5UTR\t{1}\t{2}\t.\t{3}\t.\t{4}\n".format(
               tss.seq_id, start, end,
               tss.strand, attribute_string))
+    utrs_tss.append({"strain": tss.seq_id, "start": start, "end": end,
+                     "strand": tss.strand})
 
 
 def get_5utr(tss, near_cds, utr_strain, utr_all, tas, num_utr,
-             cds_name, locus_tag, out, args_utr):
+             num, num_tss, cds_name, locus_tag, out, args_utr,
+             utrs_tss, check_cdss, pres):
     '''print and import the 5UTR information'''
+    detect = False
     if tss.strand == "+":
         start = tss.start
-        end = near_cds.start
-        length = end - start
-        detect, ta = import_utr(tss, utr_strain, utr_all,
-                                start, end, tas, length, args_utr)
+        end = near_cds.start - 1
+        length = end - start + 1
+        if str(near_cds.start) + "+" not in check_cdss:
+            detect, ta = import_utr(tss, utr_strain, utr_all,
+                                    start, end, tas, length, args_utr)
+            check_cdss.append(str(near_cds.start) + "+")
     else:
-        start = near_cds.end
+        start = near_cds.end + 1
         end = tss.end
-        length = end - start
-        detect, ta = import_utr(tss, utr_strain, utr_all,
-                                start, end, tas, length, args_utr)
+        length = end - start + 1
+        if (str(near_cds.end) + "-" not in check_cdss) or [num == num_tss]:
+            check_cdss.append(str(near_cds.end) + "-")
+            if pres["tss"] is not None:
+                detect, ta = import_utr(pres["tss"], utr_strain, utr_all,
+                                        pres["start"], pres["end"], tas,
+                                        pres["len"], args_utr)
+        pres["start"] = start
+        pres["end"] = end
+        pres["len"] = length
+        pres["tss"] = tss
     if detect:
         name_utr = '%0*d' % (5, num_utr)
         if length >= 0:
             get_print_string_5utr(num_utr, name_utr, length, tss, cds_name,
-                                  locus_tag, ta, args_utr.source, out, start, end)
+                                  locus_tag, ta, args_utr.source, out,
+                                  start, end, utrs_tss)
             num_utr += 1
     return num_utr
 
@@ -192,8 +214,9 @@ def read_file(tss_file, gff_file, ta_file, term_file):
                                              k.end, k.strand))
     if tss_file is not None:
         for entry in Gff3Parser().entries(open(tss_file)):
-            if entry.attributes["type"] != "Orphan":
-                source = True
+            if "type" in entry.attributes.keys():
+                if (entry.attributes["type"] != "Orphan"):
+                    source = True
             tsss.append(entry)
         tsss = sorted(tsss, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
     genes = sorted(genes, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
@@ -285,8 +308,11 @@ def detect_feature_5utr(feas, tss, cdss, length, check_cds):
                     tss.start <= fea.start):
                 if "locus_tag" in fea.attributes.keys():
                     locus_tag = fea.attributes["locus_tag"]
-                else:
+                elif "ID" in fea.attributes.keys():
                     locus_tag = fea.attributes["ID"]
+                else:
+                    locus_tag = (fea.feature + ":" + str(fea.start) + 
+                            "-" + str(fea.end) + "_" + fea.strand)
                 if check_cds:
                     near_cds, cds_name, check_utr = detect_cds(cdss, fea)
                 else:
@@ -299,8 +325,11 @@ def detect_feature_5utr(feas, tss, cdss, length, check_cds):
                     tss.start >= fea.end):
                 if "locus_tag" in fea.attributes.keys():
                     locus_tag = fea.attributes["locus_tag"]
-                else:
+                elif "ID" in fea.attributes.keys():
                     locus_tag = fea.attributes["ID"]
+                else:
+                    locus_tag = (fea.feature + ":" + str(fea.start) +
+                            "-" + str(fea.end) + "_" + fea.strand)
                 if check_cds:
                     near_cds, cds_name, check_utr = detect_cds(cdss, fea)
                 else:
@@ -349,6 +378,11 @@ def get_attribute_string(num_utr, length, cds, gene_name, ta, id_name, name,
                 [attribute_string,
                  "Parent=" + (feature, feature_name + str(ta.start) + "-" +
                  str(ta.end) + "_" + ta.strand)])
+    elif feature == "associated_term":
+            attribute_string = ";".join(
+                [attribute_string,
+                 "associated_term=" + (feature_name + str(ta.start) + "-" +
+                 str(ta.end) + "_" + ta.strand)])
     return attribute_string
 
 
@@ -382,9 +416,26 @@ def set_utr_strain(ta, type_, utr_strain):
         utr_strain[type_][ta.seq_id] = []
 
 
-def compare_ta(tas, genes, cdss, utr_strain, utr_all, out, args_utr):
+def check_repeat(start, end, strain, strand, utrs_tss):
+    for utr in utrs_tss:
+        if (utr["strain"] == strain) and (
+                utr["strand"] == strand):
+            if ((utr["start"] >= start) and (
+                utr["end"] <= end)) or (
+                (utr["start"] <= start) and (
+                utr["end"] >= end)) or (
+                (utr["start"] >= start) and (
+                utr["start"] <= end) and (
+                utr["end"] >= end)) or (
+                (utr["start"] <= start) and (
+                utr["end"] >= start) and (
+                utr["end"] <= end)):
+                return True
+    return False
+
+def compare_ta(tas, genes, cdss, utr_strain, utr_all, out,
+               args_utr, utrs_tss, num_utr):
     '''Comparing CDS and trancript to find the 5UTR'''
-    num_utr = 0
     for ta in tas:
         detect = False
         set_utr_strain(ta, "all", utr_strain)
@@ -401,17 +452,19 @@ def compare_ta(tas, genes, cdss, utr_strain, utr_all, out, args_utr):
                                 (ta.start <= cds.start) and (
                                 ta.end <= cds.end) and (
                                 ta.end >= cds.start)):
-                            length = cds.start - ta.start
-                            utr_strain["all"][ta.seq_id].append(length)
-                            utr_all["all"].append(length)
-                            gene_name = get_gene_name(genes, cds)
-                            string = get_attribute_string(
-                                num_utr, length, cds, gene_name, ta, "utr5",
-                                "5'UTR", "Parent", "Transcript:")
-                            detect = True
-                            start = ta.start
-                            end = cds.start
-                            break
+                            if (not check_repeat(ta.start, cds.start, ta.seq_id,
+                                                 ta.strand, utrs_tss)):
+                                length = cds.start - ta.start
+                                utr_strain["all"][ta.seq_id].append(length)
+                                utr_all["all"].append(length)
+                                gene_name = get_gene_name(genes, cds)
+                                string = get_attribute_string(
+                                    num_utr, length, cds, gene_name, ta, "utr5",
+                                    "5'UTR", "Parent", "Transcript:")
+                                detect = True
+                                start = ta.start
+                                end = cds.start - 1
+                                break
                 else:
                     if ((ta.end - cds.end) <= args_utr.length) and (
                             (ta.end - cds.end) >= 0):
@@ -420,8 +473,10 @@ def compare_ta(tas, genes, cdss, utr_strain, utr_all, out, args_utr):
                                 (ta.start >= cds.start) and (
                                 ta.start <= cds.end) and (
                                 ta.end >= cds.end)):
-                            near_cds = cds
-                            detect = True
+                            if (not check_repeat(cds.end, ta.end, ta.seq_id,
+                                                 ta.strand, utrs_tss)):
+                                near_cds = cds
+                                detect = True
         if (ta.strand == "-") and (detect):
             length = ta.end - near_cds.end
             utr_strain["all"][ta.seq_id].append(length)
@@ -430,15 +485,16 @@ def compare_ta(tas, genes, cdss, utr_strain, utr_all, out, args_utr):
             string = get_attribute_string(
                 num_utr, length, near_cds, gene_name,
                 ta, "utr5", "5'UTR", "Parent", "Transcript:")
-            start = near_cds.end
+            start = near_cds.end + 1
             end = ta.end
         if detect:
             if end - start > 0:
+                string = ";".join([string, "associated_tss=NA", "tss_type=NA"])
                 out.write("{0}\tANNOgesic\t5UTR\t{1}"
                           "\t{2}\t.\t{3}\t.\t{4}\n".format(
                               ta.seq_id, start, end, ta.strand, string))
                 num_utr += 1
-
+    return num_utr
 
 def detect_5utr(tss_file, gff_file, ta_file, out_file, args_utr):
     '''detection of 5UTR'''
@@ -450,11 +506,16 @@ def detect_5utr(tss_file, gff_file, ta_file, out_file, args_utr):
     out.write("##gff-version 3\n")
     genes, cdss, terms, tsss, tas, source = read_file(
             tss_file, gff_file, ta_file, None)
+    utrs_tss = []
+    check_cdss = []
+    pres = {"check": None, "tss": None, "start": -1, "end": -1, "len": -1}
     if (args_utr.source) and (not source):
         args_utr.source = False
     if (args_utr.base_5utr.upper() == "TSS") or (
             args_utr.base_5utr.lower() == "both"):
+        num = 0
         for tss in tsss:
+            num = num + 1
             if args_utr.source:
                 utr_datas = get_5utr_from_TSSpredator(tss, genes, cdss)
             else:
@@ -467,12 +528,16 @@ def detect_5utr(tss_file, gff_file, ta_file, out_file, args_utr):
                     utr_strain["sec"][tss.seq_id] = []
                     utr_strain["all"][tss.seq_id] = []
                 num_utr = get_5utr(tss, utr_datas["near_cds"], utr_strain,
-                                   utr_all, tas, num_utr,
+                                   utr_all, tas, num_utr, num, len(tsss),
                                    utr_datas["cds_name"], utr_datas["locus"],
-                                   out, args_utr)
-    if args_utr.base_5utr.lower() == "transcript":
-        compare_ta(tas, genes, cdss, utr_strain,
-                   utr_all, out, args_utr)
+                                   out, args_utr, utrs_tss, check_cdss, pres)
+    if (args_utr.base_5utr.lower() == "transcript") or (
+            args_utr.base_5utr.lower() == "both"):
+        num_utr = compare_ta(tas, genes, cdss, utr_strain,
+                             utr_all, out, args_utr, utrs_tss, num_utr)
+    out.close()
+    Helper().sort_gff(out_file, out_file + "sort")
+    shutil.move(out_file + "sort", out_file)
     name = (gff_file.split("/"))[-1].replace(".gff", "")
     plot(utr_all["all"], utr_all["pri"], utr_all["sec"], "_".join([name,
          "all_5utr_length.png"]), args_utr.source, "5utr", args_utr.base_5utr)
@@ -482,7 +547,6 @@ def detect_5utr(tss_file, gff_file, ta_file, out_file, args_utr):
                  utr_strain["sec"][strain],
                  "_".join([strain, "5utr_length.png"]), args_utr.source,
                  "5utr", args_utr.base_5utr)
-    out.close()
 
 
 def compare_term(ta, terms, fuzzy):
@@ -501,17 +565,17 @@ def compare_term(ta, terms, fuzzy):
 
 
 def get_3utr(ta, near_cds, utr_all, utr_strain,
-             attributes, num_utr, out, args_utr):
+             attributes, num_utr, out, args_utr, utrs_ta):
     '''print the 3UTR'''
     if ta.strand == "+":
-        start = near_cds.end
+        start = near_cds.end + 1
         end = ta.end
         length = ta.end - near_cds.end
         utr_all.append(length)
         utr_strain[ta.seq_id].append(length)
     else:
         start = ta.start
-        end = near_cds.start
+        end = near_cds.start - 1
         length = near_cds.start - ta.start
         utr_all.append(length)
         utr_strain[ta.seq_id].append(length)
@@ -530,49 +594,79 @@ def get_3utr(ta, near_cds, utr_all, utr_strain,
         id_ = "ID=" + ta.seq_id + "_utr3_" + str(num_utr)
         num_utr += 1
         attribute_string = ";".join([id_, name, attribute])
-        if args_utr.base_3utr == "transcript":
-            out.write("\t".join([ta.seq_id, "ANNOgesic", "3UTR",
-                                 str(start), str(end),
-                                 ta.score, ta.strand, ta.phase,
-                                 attribute_string]) + "\n")
-        elif args_utr.base_3utr == "both":
-            if "associated_term=NA" not in attributes:
-                out.write("\t".join([ta.seq_id, "ANNOgesic", "3UTR",
-                                     str(start), str(end),
-                                     ta.score, ta.strand, ta.phase,
-                                     attribute_string]) + "\n")
+        out.write("\t".join([ta.seq_id, "ANNOgesic", "3UTR",
+                             str(start), str(end),
+                             ta.score, ta.strand, ta.phase,
+                             attribute_string]) + "\n")
+        utrs_ta.append({"strain": ta.seq_id, "start": start, "end": end,
+                        "strand": ta.strand})
     return num_utr
 
+def get_gene_string(gene, attributes):
+    if "locus_tag" in gene.attributes.keys():
+        attributes.append("=".join(["associated_gene",
+                          gene.attributes["locus_tag"]]))
+    else:
+        gene_string = (gene.feature + ":" + str(gene.start) +
+                       "-" + str(gene.end) + "_" + gene.strand)
+        attributes.append("=".join(["associated_gene",
+                         gene_string]))
 
-def get_near_cds(cdss, genes, ta, attributes):
+
+def get_near_cds(cdss, genes, ta, attributes, utr_length):
     '''Get the associated CDS of terminator'''
-    first = True
     detect = False
     near_cds = None
     for cds in cdss:
-        if (ta.seq_id == cds.seq_id) and \
-           (ta.strand == cds.strand):
-            if first:
-                near_cds = cds
-                first = False
+        if (ta.seq_id == cds.seq_id) and (
+                ta.strand == cds.strand) and (
+                cds.feature == "CDS"):
             if ta.strand == "+":
-                if (cds.end <= ta.end) and \
-                   (cds.start >= ta.start) and \
-                   (cds.end > near_cds.end):
-                    near_cds = cds
+                if (cds.end <= ta.end) and (
+                        (ta.end - cds.end) <= utr_length) and (
+                        (ta.end - cds.end) > 0):
+                    if (near_cds == None):
+                        near_cds = cds
+                    else:
+                        if (cds.end > near_cds.end):
+                            near_cds = cds
                     detect = True
             else:
-                if (cds.start >= ta.start) and \
-                   (cds.end <= ta.end):
-                    near_cds = cds
+                if (cds.start >= ta.start) and (
+                        (cds.start - ta.start) <= utr_length) and (
+                        (cds.start - ta.start) > 0):
+                    if (near_cds == None):
+                        near_cds = cds
+                    else:
+                        if (cds.start < near_cds.start):
+                            near_cds = cds
                     detect = True
-                    break
     if detect:
+        check_gene = False
         for gene in genes:
-            if ("Parent" in near_cds.attributes.keys()):
+            if ("Parent" in near_cds.attributes.keys()) and ("ID" in gene.attributes.keys()):
                 if gene.attributes["ID"] in near_cds.attributes["Parent"].split(","):
-                    attributes.append("=".join(["associated_gene",
-                                      gene.attributes["locus_tag"]]))
+                    get_gene_string(gene, attributes)
+                    check_gene = True
+                    break
+            else:
+                if (gene.seq_id == near_cds.seq_id) and (
+                    gene.strand == near_cds.strand):
+                    if ((gene.start >= near_cds.start) and (
+                        gene.end <= near_cds.end)) or (
+                        (gene.start <= near_cds.start) and (
+                        gene.end >= near_cds.end)) or (
+                        (gene.start >= near_cds.start) and (
+                        gene.start <= near_cds.end) and (
+                        gene.end >= near_cds.end)) or (
+                        (gene.start <= near_cds.start) and (
+                        gene.end >= near_cds.start) and (
+                        gene.end <= near_cds.end)):
+                        get_gene_string(gene, attributes)
+                        check_gene = True
+                        break
+        if not check_gene:
+            attributes.append("assoicated_gene=NA")
         if "protein_id" in near_cds.attributes.keys():
             attributes.append("=".join(["associated_cds",
                               near_cds.attributes["protein_id"]]))
@@ -580,16 +674,17 @@ def get_near_cds(cdss, genes, ta, attributes):
             attributes.append("=".join(["associated_cds",
                               near_cds.attributes["locus_tag"]]))
         else:
-            attributes.append("=".join(["associated_cds",
-                              "_".join([near_cds.feature,
-                                        str(near_cds.start), str(near_cds.end),
-                                        near_cds.strand])]))
+            cds_string = (near_cds.feature + ":" + str(near_cds.start) + 
+                          "-" + str(near_cds.end) + "_" + near_cds.strand)
+            attributes.append("=".join(["associated_cds", cds_string]))
+    else:
+        near_cds = None
     return near_cds
 
 
-def compare_term_3utr(terms, cdss, genes, utr_all, utr_strain, args_utr, out):
+def compare_term_3utr(terms, cdss, genes, utr_all, utr_strain, args_utr,
+                      out, utrs_ta, num_utr):
     '''Comparing of terminator and 3UTR to get the relationship'''
-    num_utr = 0
     for term in terms:
         detect = False
         if term.seq_id not in utr_strain.keys():
@@ -602,24 +697,28 @@ def compare_term_3utr(terms, cdss, genes, utr_all, utr_strain, args_utr, out):
                             ((term.end - cds.end) <= args_utr.length) or (
                             ((term.start - cds.end) <= args_utr.length) and (
                             term.start >= cds.end))):
-                        detect = True
-                        near_cds = cds
+                        if (not check_repeat(cds.end, term.end, cds.seq_id, 
+                            cds.strand, utrs_ta)):
+                            detect = True
+                            near_cds = cds
                 else:
                     if (term.start <= cds.start) and (
                             ((cds.start - term.start) <= args_utr.length) or (
                             ((cds.start - term.end) <= args_utr.length) and (
                             cds.start >= term.end))):
-                        length = term.start - cds.start
-                        utr_strain[term.seq_id].append(length)
-                        utr_all.append(length)
-                        gene_name = get_gene_name(genes, cds)
-                        string = get_attribute_string(
-                            num_utr, length, cds, gene_name, term, "utr3",
-                            "3'UTR", "associated_term", "Terminator:")
-                        detect = True
-                        start = term.start
-                        end = cds.start
-                        break
+                        if (not check_repeat(term.start, cds.start, cds.seq_id,
+                            cds.strand, utrs_ta)):
+                            length = term.start - cds.start
+                            utr_strain[term.seq_id].append(length)
+                            utr_all.append(length)
+                            gene_name = get_gene_name(genes, cds)
+                            string = get_attribute_string(
+                                num_utr, length, cds, gene_name, term, "utr3",
+                                "3'UTR", "associated_term", "Terminator:")
+                            detect = True
+                            start = term.start
+                            end = cds.start - 1
+                            break
             if (term.strand == "+") and detect:
                 length = term.end - near_cds.end
                 utr_strain[term.seq_id].append(length)
@@ -629,14 +728,15 @@ def compare_term_3utr(terms, cdss, genes, utr_all, utr_strain, args_utr, out):
                     num_utr, length, near_cds, gene_name, term, "utr3",
                     "3'UTR", "associated_term", "Terminator:")
                 detect = True
-                start = near_cds.end
+                start = near_cds.end + 1
                 end = term.end
         if detect:
             if end - start > 0:
-                out.write("{0}\tANNOgesic\t5UTR\t{1}\t"
+                out.write("{0}\tANNOgesic\t3UTR\t{1}\t"
                           "{2}\t.\t{3}\t.\t{4}\n".format(
                               term.seq_id, start, end, term.strand, string))
                 num_utr += 1
+    return num_utr
 
 
 def detect_3utr(ta_file, gff_file, term_file, out_file, args_utr):
@@ -649,6 +749,7 @@ def detect_3utr(ta_file, gff_file, term_file, out_file, args_utr):
     out.write("##gff-version 3\n")
     genes, cdss, terms, tsss, tas, args_utr.source = read_file(
             None, gff_file, ta_file, term_file)
+    utrs_ta = []
     if (args_utr.base_3utr == "transcript") or (
             args_utr.base_3utr == "both"):
         for ta in tas:
@@ -665,20 +766,23 @@ def detect_3utr(ta_file, gff_file, term_file, out_file, args_utr):
                                   str(term.end) + "_" + term.strand]))
                 else:
                     attributes.append("=".join(["associated_term", "NA"]))
-            near_cds = get_near_cds(cdss, genes, ta, attributes)
+            near_cds = get_near_cds(cdss, genes, ta, attributes, args_utr.length)
             if ta.seq_id != pre_seq_id:
                 pre_seq_id = ta.seq_id
                 utr_strain[ta.seq_id] = []
             if near_cds is not None:
                 num_utr = get_3utr(ta, near_cds, utr_all, utr_strain,
-                                   attributes, num_utr, out, args_utr)
-    else:
-        compare_term_3utr(terms, cdss, genes, utr_all,
-                          utr_strain, args_utr, out)
+                                   attributes, num_utr, out, args_utr, utrs_ta)
+    if (args_utr.base_3utr == "terminator") or (
+            args_utr.base_3utr == "both"):
+        num_utr = compare_term_3utr(terms, cdss, genes, utr_all, utr_strain,
+                                    args_utr, out, utrs_ta, num_utr)
+    out.close()
+    Helper().sort_gff(out_file, out_file + "sort")
+    shutil.move(out_file + "sort", out_file)
     name = (gff_file.split("/"))[-1].replace(".gff", "")
     plot(utr_all, None, None, "_".join([name, "all_3utr_length.png"]),
          None, "3utr", "3utr")
-    out.close()
     if len(utr_strain) > 1:
         for strain in utr_strain.keys():
             plot(utr_strain[strain], None, None,
