@@ -149,8 +149,6 @@ class TranscriptDetection(object):
         for strain in strains:
             out = os.path.join(self.gff_outfolder, "_".join([
                 strain, "transcript", type_ + ".gff"]))
-            print(os.path.join(args_tran.out_folder,
-                                 "_".join([strain, type_])))
             self.helper.sort_gff(os.path.join(args_tran.out_folder,
                                  "_".join([strain, type_])), out)
             os.remove(os.path.join(args_tran.out_folder,
@@ -309,7 +307,7 @@ class TranscriptDetection(object):
             strains = self._for_one_wig("tex_notex", args_tran)
         self._for_two_wigs(strains, args_tran, log)
         tas = []
-        if "none" not in args_tran.modify:
+        if ("none" not in args_tran.modify) and (args_tran.gffs is not None):
             for gff in os.listdir(args_tran.gffs):
                 if gff.endswith(".gff"):
                     self.helper.sort_gff(os.path.join(args_tran.gffs, gff),
